/* eslint-env node */
require('@rushstack/eslint-patch/modern-module-resolution');

module.exports = {
  rules: {
    //// vue
    // strongly recommended
    'vue/attribute-hyphenation': ['warn', 'never'],
    'vue/v-on-event-hyphenation': ['warn', 'never'],
    // recommended
    'vue/component-tags-order': ['warn', {order: ['script', 'template', 'style']}],
    // uncategorized
    'vue/block-lang': ['error', {script: {lang: 'ts'}}],
    'vue/block-tag-newline': ['warn', {singleline: 'always'}],
    'vue/component-api-style': ['error', ['script-setup']],
    'vue/component-name-in-template-casing': 'warn',
    'vue/component-options-name-casing': 'warn',
    'vue/custom-event-name-casing': ['error', 'camelCase'],
    'vue/define-macros-order': 'warn',
    'vue/html-button-has-type': 'error',
    'vue/html-comment-content-newline': 'warn',
    'vue/html-comment-content-spacing': 'warn',
    'vue/html-comment-indent': 'warn',
    'vue/next-tick-style': 'warn',
    'vue/no-boolean-default': 'error',
    'vue/no-duplicate-attr-inheritance': 'error',
    'vue/no-multiple-objects-in-class': 'warn',
    'vue/no-static-inline-styles': 'warn',
    'vue/no-template-target-blank': 'warn',
    'vue/no-this-in-before-route-enter': 'error',
    'vue/no-undef-components': 'error',
    'vue/no-undef-properties': 'error',
    'vue/no-unused-properties': 'warn',
    'vue/no-unused-refs': 'warn',
    'vue/no-useless-mustaches': 'warn',
    'vue/no-useless-v-bind': 'warn',
    'vue/no-v-text': 'error',
    'vue/padding-line-between-blocks': 'warn',
    'vue/prefer-prop-type-boolean-first': 'error',
    'vue/prefer-separate-static-class': 'warn',
    'vue/prefer-true-attribute-shorthand': 'warn',
    'vue/require-emit-validator': 'error',
    'vue/script-indent': 'warn',
    'vue/v-for-delimiter-style': 'warn',
    'vue/v-on-function-call': 'warn',
    // extensions
    'vue/no-constant-condition': 'warn',
    'vue/no-empty-pattern': 'warn',
    'vue/no-irregular-whitespace': 'warn',
    'vue/no-loss-of-precision': 'warn',
    'vue/no-sparse-arrays': 'warn',
    //// eslint
    // possible problems
    'array-callback-return': 'warn',
    'no-await-in-loop': 'warn',
    'no-constant-binary-expression': 'warn',
    'no-constructor-return': 'error',
    'no-duplicate-imports': 'warn',
    'no-promise-executor-return': 'error',
    'no-self-compare': 'warn',
    'no-template-curly-in-string': 'warn',
    'no-unmodified-loop-condition': 'warn',
    'no-unreachable-loop': 'warn',
    'no-unused-private-class-members': 'warn',
    'no-use-before-define': 'off',
    '@typescript-eslint/no-use-before-define': 'error',
    'require-atomic-updates': 'warn',
    // suggestions
    'accessor-pairs': 'error',
    'arrow-body-style': 'warn',
    'block-scoped-var': 'error',
    'camelcase': 'warn',
    'vue/camelcase': 'warn',
    'curly': 'warn',
    'default-case': 'error',
    'default-case-last': 'error',
    'default-param-last': 'off',
    '@typescript-eslint/default-param-last': 'error',
    'dot-notation': 'warn',
    'eqeqeq': 'warn',
    'vue/eqeqeq': 'warn',
    'func-name-matching': 'error',
    'func-names': ['warn', 'as-needed'],
    'func-style': 'warn',
    'init-declarations': 'off',
    '@typescript-eslint/init-declarations': 'warn',
    'new-cap': 'error',
    'no-alert': 'error',
    'no-else-return': 'warn',
    'no-eval': 'warn',
    'no-extend-native': 'warn',
    'no-extra-bind': 'warn',
    'no-extra-label': 'warn',
    'no-extra-semi': 'off',
    '@typescript-eslint/no-extra-semi': 'warn',
    'no-floating-decimal': 'warn',
    'no-implicit-coercion': ['warn', {boolean: false}],
    'no-implicit-globals': 'warn',
    'no-invalid-this': 'off',
    '@typescript-eslint/no-invalid-this': 'error',
    'no-label-var': 'warn',
    'no-lonely-if': 'warn',
    'no-loop-func': 'off',
    '@typescript-eslint/no-loop-func': 'error',
    'no-mixed-operators': 'warn',
    'no-multi-assign': 'error',
    'no-nested-ternary': 'warn',
    'no-new': 'error',
    'no-new-func': 'error',
    'no-new-object': 'error',
    'no-new-wrappers': 'error',
    'no-param-reassign': 'error',
    'no-proto': 'error',
    'no-redeclare': 'off',
    '@typescript-eslint/no-redeclare': 'error',
    'no-return-assign': 'error',
    'no-sequences': 'warn',
    'no-shadow': 'off',
    '@typescript-eslint/no-shadow': 'error',
    'no-throw-literal': 'warn',
    'no-unneeded-ternary': 'warn',
    'no-unused-expressions': 'warn',
    '@typescript-eslint/no-unused-expressions': 'warn',
    'no-useless-call': 'warn',
    'no-useless-computed-key': 'warn',
    'no-useless-constructor': 'warn',
    'no-useless-rename': 'warn',
    'no-useless-return': 'warn',
    'no-void': 'error',
    'object-shorthand': 'warn',
    'vue/object-shorthand': 'warn',
    'one-var': ['warn', 'never'],
    'one-var-declaration-per-line': 'warn',
    'prefer-arrow-callback': 'warn',
    'prefer-named-capture-group': 'warn',
    'prefer-object-spread': 'warn',
    'prefer-promise-reject-errors': 'warn',
    'prefer-rest-params': 'warn',
    'prefer-spread': 'warn',
    'prefer-template': 'warn',
    'vue/prefer-template': 'warn',
    'quote-props': ['warn', 'consistent-as-needed'],
    'radix': 'warn',
    'require-await': 'warn',
    'yoda': 'warn',
    // Layout & Formatting
    'array-bracket-newline': ['warn', {multiline: true}],
    'vue/array-bracket-newline': ['warn', {multiline: true}],
    'array-bracket-spacing': 'warn',
    'vue/array-bracket-spacing': 'warn',
    'array-element-newline': ['warn', 'consistent'],
    'arrow-parens': ['warn', 'as-needed'],
    'arrow-spacing': 'warn',
    'vue/arrow-spacing': 'warn',
    'block-spacing': 'warn',
    'vue/block-spacing': 'warn',
    'brace-style': 'off',
    '@typescript-eslint/brace-style': 'warn',
    'vue/brace-style': 'warn',
    'comma-dangle': 'off',
    '@typescript-eslint/comma-dangle': ['warn', 'always-multiline'],
    'vue/comma-dangle': ['warn', 'always-multiline'],
    'comma-spacing': 'off',
    '@typescript-eslint/comma-spacing': 'warn',
    'vue/comma-spacing': 'warn',
    'comma-style': 'warn',
    'vue/comma-style': 'warn',
    'computed-property-spacing': 'warn',
    'eol-last': 'warn',
    'func-call-spacing': 'off',
    '@typescript-eslint/func-call-spacing': 'warn',
    'vue/func-call-spacing': 'warn',
    'function-call-argument-newline': ['warn', 'consistent'],
    'function-paren-newline': ['warn', 'multiline'],
    'implicit-arrow-linebreak': 'warn',
    'indent': ['warn', 2],
    'key-spacing': ['warn', {mode: 'minimum'}],
    'vue/key-spacing': ['warn', {mode: 'minimum'}],
    'keyword-spacing': 'off',
    '@typescript-eslint/keyword-spacing': 'warn',
    'vue/keyword-spacing': 'warn',
    'max-statements-per-line': 'warn',
    'new-parens': 'warn',
    'no-extra-parens': 'off',
    '@typescript-eslint/no-extra-parens': 'warn',
    'no-multi-spaces': 'warn',
    'no-trailing-spaces': 'warn',
    'no-whitespace-before-property': 'warn',
    'object-curly-newline': ['warn', {consistent: true}],
    'vue/object-curly-newline': ['warn', {consistent: true}],
    'object-curly-spacing': 'off',
    '@typescript-eslint/object-curly-spacing': 'warn',
    'vue/object-curly-spacing': 'warn',
    'object-property-newline': 'warn',
    'vue/object-property-newline': 'warn',
    'operator-linebreak': 'warn',
    'vue/operator-linebreak': 'warn',
    'padding-line-between-statements': 'off',
    '@typescript-eslint/padding-line-between-statements': [
      'error',
      {
        blankLine: 'always',
        prev: 'import',
        next: '*',
      },
      {
        blankLine: 'any',
        prev: 'import',
        next: 'import',
      },
      {
        blankLine: 'always',
        prev: '*',
        next: 'return',
      },
    ],
    'quotes': 'off',
    '@typescript-eslint/quotes': ['warn', 'single'],
    'semi': 'off',
    '@typescript-eslint/semi': ['warn', 'always'],
    'semi-spacing': 'warn',
    'semi-style': 'warn',
    'space-before-function-paren': 'off',
    '@typescript-eslint/space-before-function-paren': ['warn', 'never'],
    'space-in-parens': 'warn',
    'vue/space-in-parens': 'warn',
    'space-infix-ops': 'off',
    '@typescript-eslint/space-infix-ops': 'warn',
    'vue/space-infix-ops': 'warn',
    'space-unary-ops': 'warn',
    'vue/space-unary-ops': 'warn',
    'switch-colon-spacing': 'warn',
    'template-curly-spacing': 'warn',
    'vue/template-curly-spacing': 'warn',
    'wrap-iife': 'warn',
    //// typescript-eslint
    '@typescript-eslint/array-type': 'warn',
    '@typescript-eslint/ban-ts-comment': 'warn',
    '@typescript-eslint/consistent-generic-constructors': 'warn',
    '@typescript-eslint/consistent-indexed-object-style': 'warn',
    '@typescript-eslint/consistent-type-assertions': 'warn',
    '@typescript-eslint/consistent-type-definitions': 'warn',
    '@typescript-eslint/consistent-type-imports': 'warn',
    '@typescript-eslint/explicit-function-return-type': ['error'],
    '@typescript-eslint/member-delimiter-style': 'warn',
    '@typescript-eslint/method-signature-style': 'warn',
    '@typescript-eslint/no-confusing-non-null-assertion': 'warn',
    'no-dupe-class-members': 'off',
    '@typescript-eslint/no-dupe-class-members': 'error',
    '@typescript-eslint/no-duplicate-enum-values': 'error',
    '@typescript-eslint/no-extraneous-class': 'error',
    '@typescript-eslint/no-useless-empty-export': 'warn',
    '@typescript-eslint/type-annotation-spacing': 'warn',

    // disable rules that need type information
    '@typescript-eslint/no-base-to-string': 'off',
    '@typescript-eslint/no-meaningless-void-operator': 'off',
    '@typescript-eslint/no-unnecessary-boolean-literal-compare': 'off',
    '@typescript-eslint/no-unnecessary-condition': 'off',
    '@typescript-eslint/no-unnecessary-type-arguments': 'off',
    '@typescript-eslint/non-nullable-type-assertion-style': 'off',
    '@typescript-eslint/prefer-includes': 'off',
    '@typescript-eslint/prefer-nullish-coalescing': 'off',
    '@typescript-eslint/prefer-reduce-type-parameter': 'off',
    '@typescript-eslint/prefer-return-this-type': 'off',
    '@typescript-eslint/prefer-string-starts-ends-with': 'off',
    '@typescript-eslint/dot-notation': 'off',
    '@typescript-eslint/no-throw-literal': 'off',
    '@typescript-eslint/prefer-optional-chain': 'off',
  },
};
